@@ -6,6 +6,7 @@ pub use dependency_groups::*;
 pub use dry_run::*;
 pub use editable::*;
 pub use env_file::*;
+pub use excludes::*;
 pub use export_format::*;
 pub use extras::*;
 pub use hash::*;
@@ -30,6 +31,7 @@ mod dependency_groups;
 mod dry_run;
 mod editable;
 mod env_file;
+mod excludes;
 mod export_format;
 mod extras;
 mod hash;
