@@ -314,6 +314,7 @@ impl SitePackages {
         requirements: &[UnresolvedRequirementSpecification],
         constraints: &[NameRequirementSpecification],
         overrides: &[UnresolvedRequirementSpecification],
+        installation: InstallationStrategy,
         markers: &ResolverMarkerEnvironment,
         tags: &Tags,
         config_settings: &ConfigSettings,
@@ -404,6 +405,7 @@ impl SitePackages {
             requirements.iter().map(Cow::as_ref),
             constraints.iter().map(|constraint| &constraint.requirement),
             overrides.iter().map(Cow::as_ref),
+            installation,
             markers,
             tags,
             config_settings,
@@ -419,6 +421,7 @@ impl SitePackages {
         requirements: impl ExactSizeIterator<Item = &'a Requirement>,
         constraints: impl Iterator<Item = &'a Requirement>,
         overrides: impl Iterator<Item = &'a Requirement>,
+        installation: InstallationStrategy,
         markers: &ResolverMarkerEnvironment,
         tags: &Tags,
         config_settings: &ConfigSettings,
@@ -482,6 +485,7 @@ impl SitePackages {
                             name,
                             distribution,
                             &requirement.source,
+                            installation,
                             tags,
                             config_settings,
                             config_settings_package,
@@ -504,6 +508,7 @@ impl SitePackages {
                                 name,
                                 distribution,
                                 &constraint.source,
+                                installation,
                                 tags,
                                 config_settings,
                                 config_settings_package,
@@ -560,6 +565,27 @@ impl SitePackages {
     }
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub enum InstallationStrategy {
+    /// A permissive installation strategy, which accepts existing installations even if the source
+    /// type differs, as in the `pip` and `uv pip` CLIs.
+    ///
+    /// In this strategy, packages that are already installed in the environment may be reused if
+    /// they implicitly match the requirements. For example, if the user installs `./path/to/idna`,
+    /// then runs `uv pip install anyio` (which depends on `idna`), the existing `idna` installation
+    /// will be reused if its version matches the requirement, even though it was installed from a
+    /// path and is being implicitly requested from a registry.
+    Permissive,
+
+    /// A strict installation strategy, which requires that existing installations match the source
+    /// type, as in the `uv sync` CLI.
+    ///
+    /// This strategy enforces that the installation source must match the requirement source.
+    /// It prevents reusing packages that were installed from different sources, ensuring
+    /// declarative and reproducible environments.
+    Strict,
+}
+
 /// We check if all requirements are already satisfied, recursing through the requirements tree.
 #[derive(Debug)]
 pub enum SatisfiesResult {
