@@ -575,8 +575,8 @@ fn help_subsubcommand() {
               executables are also installed.
               
               Alternative Python variants will still include their tag. For example, installing
-              3.13+freethreaded with `--default` will include in `python3t` and `pythont`, not `python3`
-              and `python`.
+              3.13+freethreaded with `--default` will include `python3t` and `pythont` instead of
+              `python3` and `python`.
               
               If multiple Python versions are requested, uv will exit with an error.
 
